# Demo 08

This demo will in the end have:

- A functioning database model
- Ways to seed the database from a YAML dataset
- Ways to display content from the database dynamically in the browser

## Steps:

Assess the files in the project
Each file you need to modify contains a **TODO** comment

```txt
examples                    - with html output examples (you can compare your rendered html to these)
prisma                      - prisma data folder with migrations and
public                      - static content (css) that express serves
src                         - contains TypeScript code
 ├ data-loading
 │  └ loading.ts            - for loading and parsing the data.yaml file
 ├ data-processing
 │  └ seed-from-file.ts     - contains logic for seeding the DTOs from file
 ├ seed
 │  └ seed.ts               - contains the seed script
 ├ types
 │  └ yaml-types.ts         - contains the yaml DTOs
 └ index.ts                 - Webserver (express app) entry-point
views - contains templates for html rendering
.env.example                - example DB connection string
.gitignore
compose.yml                 - compose file with a pre-defined postgres container
data.yml                    - database seed file
package-lock.json
package.json
README.md
tsconfig.json
```

- either setup an `sqlite` database - for school computers, or run your `compose` file from this folder to get a postgres database running

---

### Seed from file - TASK 1

- The YAML parsing of the file data to TS DTOs is already implemented and used by the `seed` script
- The `seed` script is already implemented in the `src/seed/seed.ts`
- Your task is to model the database in Prisma from the provided ERD
  - modify the provided `prisma/schema.prisma` - model the DB from the provided ERD (`prisma/erd.puml`)
  - follow the names of the properties precisely
  - run a migration to propagate the schema to your db
  - implement the function in `src/data-processing/seed-from-file.ts` file:

```ts
export const seedArtists = async (artists: ArtistDTO[]): Promise<Result<{}>> => {...};
```

This function will be called by the `seed` script and if written properly, the database will be seeded with it.

_note: write auxiliary functions, one function for seeding can become really long and messy_
_note 2: you will need `(for const X of Y)` loops to be able to `await` creating the db objects when looping (as opposed to traditionally using functional methods)_

You can invoke the `seed` script by running:

```sh
npm run seed
# The expected output:
[1648896941994] seeding from the data file has been completed
# the number will differ - it is the current timestamp
```

- Run the seed and then check your database - you can do so in Prisma studio (similar to adminer) by running:

```sh
npx prisma studio
```

---

### Server-side rendering - TASK 2

Create routes for an Express router that handles the `/artist` route (located in the `src/routing/artist.ts`)
The router is already imported in the `index.ts` file, just implement the functionality
You can get inspired by the `index.ts` router

- Implement the `/artist` route (`/` in that router) which will render an error message (no artist was specified)
  - The renderer for `artist` page expects the error as follows:

```ts
res.render("artist.twig", {
    error: "Your error message",
    pageTitle: "Error shown in the browser tab (page <title></title> node)",
    mainTitle: "Error shown on the webpage",
});
```

After you render an error, exit the function (by calling `return`)

- Implement the `/artist/id` route (`/:id` in the router) where `id` is an id of the artist from the db
  - Parse the id parameter as a number, retrieve data from the db via Prisma
    - handle the possible errors (id is not a number, artist doesnt exist, artist has no albums...)
    - these errors will cause the function to render an error in the same manner as the previous route did
  - If you manage to obtain valid data, pass them to the renderer in this manner:

```ts
res.render("artist.twig", {
    artist: anArtistObjectFromPrisma,
    albums: arrayOfAlbumObjectsFromPrisma,
    pageTitle: `${anArtistObjectFromPrisma?.name}'s albums`,
    mainTitle: anArtistObjectFromPrisma?.name,
});
```

The templates are already pre-defined, so you don't have to learn the templating as well during this seminar.

You can run the express app by the pre-defined `start` script:

```sh
npm start
```

---

### Rendering songs for artist's albums - OPTIONAL TASK @home

If you wish, you can continue with this project and create a page that will render an album:

Recommended Steps:

- Create a router for the `/albums` route and register it in the `index.ts` file (put it in the separate file in the `src/routes` folder)
- Handle the empty route as well as the route with the album id parameter
- Handle the http parameter parsing and prisma queries accordingly - handle edge cases
- Create an `album.twig` file where you'll display the songs from that album
  - You can get inpiration fromt the `artist.twig` file
  - You might need to create your own html and css elements, or you can just use the same elements - it will look funky though
- Modify `artist.twig` album anchor tags (`<a>`) to point at the desired location - `/albums/yourAlbumID`

---

## Notes

If you wish, you can expand the dataset yourself (follow the modelled data) and re-seed the database

if you need to reset the database, you can to do by running:

```sh
npx prisma migrate reset
```
