import express from "express";
import dotenv from "dotenv";
import artistRouter from "./routing/artist";
import { PrismaClient } from "@prisma/client";

dotenv.config();

const app = express();
const router = express.Router();
const prisma = new PrismaClient();

// set Twig as our HTML renderer
app.set("view engine", "twig");

app.use(router);

// This section is optional and used to configure twig.
app.set("twig options", {
  allow_async: true, // Allow asynchronous compiling
  strict_variables: false,
});

// nesting the routers for artists
router.use("/artist", artistRouter);

// routing done via express router
router.use("/", (_req, _res, next) => {
  console.log(`[${Date.now()}] Incoming Request`);
  next();
});

router.get("/", async (_, res) => {
  try {
    const artistsData = await prisma.artist.findMany();
    res.render("index.twig", {
      artists: artistsData,
      pageTitle: "Spotify",
      mainTitle: "Artists",
    });
  } catch (e) {
    res.render("index.twig", {
      artists: [],
      pageTitle: "Spotify",
      mainTitle: "Artists",
    });
  }
});

// serve static files
app.use(express.static("public"));

// express listener
app.listen(process.env["PORT"] || 3000, () => {
  console.log(
    `Express app is listening at http://localhost:${
      process.env["PORT"] || 3000
    }`
  );
});
