import { loadData, parseData } from "../data-loading/loading";
import { seedArtists } from "../data-processing/seed-from-file";
import type { ArtistDTO } from "../types/yaml-types";

const seed = async () => {
  const data = loadData("./data.yml");

  // no parsed data
  if (data.isErr) {
    console.log(
      `Could not read the data from the specified file: ${data.error.message}`
    );
    return;
  }

  const parsedData = parseData(data.value);

  // no parsed data
  if (parsedData.isErr) {
    console.log(`Could not parse the data: ${parsedData.error}`);
    return;
  }

  const artists: ArtistDTO[] = parsedData.value.artists;

  const seededArtists = await seedArtists(artists);

  if (seededArtists.isErr) {
    console.log(
      `Could not seed artists to the database: ${seededArtists.error.message}`
    );
    return;
  }

  console.log(`[${Date.now()}] seeding from the data file has been completed`);
};

seed().catch((e) => {
  throw e;
});
